/*
 * Copyright 2019 Confluent Inc.
 */

package io.confluent.connect.secretregistry;

import static io.confluent.kafka.secretregistry.storage.KafkaSecretRegistry.getUriInfoForIdentity;

import java.util.Locale;
import java.util.Map;
import java.util.concurrent.locks.Condition;
import java.util.concurrent.locks.ReentrantLock;

import io.confluent.connect.secretregistry.util.Version;
import io.confluent.kafka.secretregistry.exceptions.SecretRegistryException;
import io.confluent.kafka.secretregistry.exceptions.SecretRegistryInitializationException;
import io.confluent.kafka.secretregistry.rest.SslFactory;
import io.confluent.kafka.secretregistry.rest.resources.PathKeyResource;
import io.confluent.kafka.secretregistry.rest.resources.PathKeyVersionResource;
import io.confluent.kafka.secretregistry.rest.resources.PathResource;
import io.confluent.kafka.secretregistry.storage.KafkaSecretRegistry;
import io.confluent.kafka.secretregistry.storage.UriInfo;
import io.confluent.security.auth.client.RestAuthorizer;
import io.confluent.security.authorizer.Scope;

import org.apache.kafka.connect.errors.ConnectException;
import org.apache.kafka.connect.health.ConnectClusterState;
import org.apache.kafka.connect.rest.ConnectRestExtension;
import org.apache.kafka.connect.rest.ConnectRestExtensionContext;
import org.apache.kafka.connect.runtime.WorkerConfig;
import org.apache.kafka.connect.runtime.distributed.DistributedConfig;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class ConnectSecretRegistryExtension implements ConnectRestExtension {

  private static final Logger log = LoggerFactory.getLogger(ConnectSecretRegistryExtension.class);

  public static final String CONNECT_CLUSTER_TYPE = "connect-cluster";
  private static final String PROTOCOL_HTTP = "http";
  private static final String PROTOCOL_HTTPS = "https";
  private static final String STANDALONE_CLUSTER = "STANDALONE";

  private static ReentrantLock lock = new ReentrantLock();
  private static Condition isInit = lock.newCondition();
  private static KafkaSecretRegistry secretRegistry;

  private WorkerConfig workerConfig;
  private Map<String, ?> configs;

  @Override
  @SuppressWarnings("unchecked")
  public void configure(Map<String, ?> configs) {
    this.configs = configs;

    // The configs parameter here is derived from the worker's properties file, which uses Strings
    // for its keys and values. So, the unchecked cast here should be fine.
    workerConfig = new ConcreteWorkerConfig((Map<String, String>) configs);
  }

  private void initSecretRegistry(KafkaSecretRegistry secretRegistry)
      throws SecretRegistryException {
    String host = workerConfig.getString(WorkerConfig.REST_ADVERTISED_HOST_NAME_CONFIG);
    Integer port = workerConfig.getInt(WorkerConfig.REST_ADVERTISED_PORT_CONFIG);
    if (port == null) {
      port = workerConfig.getInt(WorkerConfig.REST_PORT_CONFIG);
    }
    UriInfo uriInfo = getUriInfoForIdentity(
        host,
        port,
        workerConfig.getList(WorkerConfig.LISTENERS_CONFIG),
        determineAdvertisedProtocol()
    );
    SslFactory sslFactory = new SslFactory(workerConfig.originalsStrings());
    secretRegistry.initRest(sslFactory, uriInfo);
  }

  private String determineAdvertisedProtocol() {
    String advertisedSecurityProtocol =
        workerConfig.getString(WorkerConfig.REST_ADVERTISED_LISTENER_CONFIG);
    if (advertisedSecurityProtocol == null) {
      String listeners = (String) workerConfig.originals().get(WorkerConfig.LISTENERS_CONFIG);

      if (listeners == null) {
        return PROTOCOL_HTTP;
      } else {
        listeners = listeners.toLowerCase(Locale.ENGLISH);
      }

      if (listeners.contains(String.format("%s://", PROTOCOL_HTTP))) {
        return PROTOCOL_HTTP;
      } else if (listeners.contains(String.format("%s://", PROTOCOL_HTTPS))) {
        return PROTOCOL_HTTPS;
      } else {
        return PROTOCOL_HTTP;
      }
    } else {
      return advertisedSecurityProtocol.toLowerCase(Locale.ENGLISH);
    }
  }

  @Override
  public void register(ConnectRestExtensionContext restExtensionContext) {
    try {
      KafkaSecretRegistry secretRegistry = secretRegistry();
      initSecretRegistry(secretRegistry);

      Scope scope = determineScope(configs, restExtensionContext.clusterState());
      log.info("Registering RBAC authorizer on cluster with scope '{}'", scope);

      RestAuthorizer restAuthorizer = new RestAuthorizer();
      restAuthorizer.configure(workerConfig.originals());

      ConnectSecretRegistryFilter filter = new ConnectSecretRegistryFilter(
          scope,
          restAuthorizer,
          restExtensionContext.clusterState()
      );

      restExtensionContext
          .configurable()
          .register(filter);

      restExtensionContext
          .configurable()
          .register(new PathResource(secretRegistry));
      restExtensionContext
          .configurable()
          .register(new PathKeyResource(secretRegistry));
      restExtensionContext
          .configurable()
          .register(new PathKeyVersionResource(secretRegistry));

    } catch (SecretRegistryException e) {
      log.error("Error initializing the secret registry", e);
      close();
      throw new ConnectException("Error initializing the secret registry", e);
    }
  }

  static Scope determineScope(Map<String, ?> configs, ConnectClusterState clusterState) {
    String kafkaClusterId = clusterState.clusterDetails().kafkaClusterId();

    Object groupId = configs.get(DistributedConfig.GROUP_ID_CONFIG);
    String clusterId = groupId != null ? groupId.toString() : STANDALONE_CLUSTER;

    return new Scope.Builder()
            .withKafkaCluster(kafkaClusterId)
            .withCluster(CONNECT_CLUSTER_TYPE, clusterId)
            .build();
  }

  @Override
  public String version() {
    return Version.getVersion();
  }

  @Override
  public void close() {
    closeSecretRegistry();
  }

  public static void configureSecretRegistry(KafkaSecretRegistry registry) {
    lock.lock();
    try {
      secretRegistry = registry;
      isInit.signal();
    } finally {
      lock.unlock();
    }
  }

  protected static KafkaSecretRegistry secretRegistry()
      throws SecretRegistryInitializationException {
    lock.lock();
    try {
      while (secretRegistry == null) {
        isInit.await();
      }
    } catch (InterruptedException e) {
      Thread.currentThread().interrupt();
      String errMsg = "Interrupted waiting for initialization of InternalSecretConfigProvider";
      log.error(errMsg);
      throw new SecretRegistryInitializationException(errMsg);
    } finally {
      lock.unlock();
    }
    return secretRegistry;
  }

  private static void closeSecretRegistry() {
    lock.lock();
    try {
      if (secretRegistry != null) {
        secretRegistry.close();
        secretRegistry = null;
      }
    } finally {
      lock.unlock();
    }
  }

  // Workaround to enable creation of a generic WorkerConfig since the baseConfigDef() method is
  // protected
  private static class ConcreteWorkerConfig extends WorkerConfig {
    public ConcreteWorkerConfig(Map<String, String> props) {
      super(baseConfigDef(), props);
    }
  }
}
